@@ -46,6 +46,7 @@
 typedef struct param_t {
     /* Configurable parameters */
     char           *acCmd; /* Commandline to spawn */
+    char           *acFiletmp;
     int             fTitleDisplayed;
     int             fTitleDisplayedtmp;
     char           *acTitle;
@@ -82,7 +83,7 @@ typedef struct monitor_t {
 
 typedef struct genmon_t {
     XfcePanelPlugin    *plugin;
-    unsigned int        iTimerId; /* Cyclic update */
+    guint               iTimerId; /* Cyclic update */
     struct conf_t       oConf;
     struct monitor_t    oMonitor;
     char               *acValue; /* Commandline resulting string */
@@ -280,6 +281,7 @@ static int DisplayCmdOutput (struct genmon_t *p_poPlugin)
         /* Get the text */
         char *buf = g_strndup (begin + 5, end - begin - 5);
         gtk_label_set_markup (GTK_LABEL (poMonitor->wValue), buf);
+        gtk_label_set_justify (GTK_LABEL (poMonitor->wValue), GTK_JUSTIFY_CENTER);
         
         /* Test if the result has a clickable Value (button) */
         begin=strstr(p_poPlugin->acValue, "<txtclick>");
@@ -288,6 +290,7 @@ static int DisplayCmdOutput (struct genmon_t *p_poPlugin)
         {
             /* Add the text to the button label too*/
             gtk_label_set_markup (GTK_LABEL (poMonitor->wValButtonLabel), buf);
+            gtk_label_set_justify (GTK_LABEL (poMonitor->wValButtonLabel), GTK_JUSTIFY_CENTER);
 
             /* Get the command path */
             g_free (poMonitor->onValClickCmd);
@@ -368,27 +371,24 @@ static int DisplayCmdOutput (struct genmon_t *p_poPlugin)
 
 /**************************************************************/
 
-static gboolean SetTimer (void *p_pvPlugin)
-/* Recurrently update the panel-docked monitor through a timer */
-/* Warning : should not be called directly (except the 1st time) */
-/* To avoid multiple timers */
+static gboolean Timer (gpointer user_data)
 {
-    struct genmon_t *poPlugin = (genmon_t *) p_pvPlugin;
-    struct param_t *poConf = &(poPlugin->oConf.oParam);
+    struct genmon_t *poPlugin = user_data;
 
     DBG("\n");
-
     DisplayCmdOutput (poPlugin);
-
-    if (poPlugin->iTimerId == 0)
-    {
-        poPlugin->iTimerId = g_timeout_add (poConf->iPeriod_ms,
-            (GSourceFunc) SetTimer, poPlugin);
-        return FALSE;
-    }
-
     return TRUE;
-}/* SetTimer() */
+}
+
+static void SetTimer (struct genmon_t *poPlugin)
+{
+    struct param_t *poConf = &poPlugin->oConf.oParam;
+
+    if (poPlugin->iTimerId)
+        g_source_remove (poPlugin->iTimerId);
+
+    poPlugin->iTimerId = g_timeout_add (poConf->iPeriod_ms, Timer, poPlugin);
+}
 
 /**************************************************************/
 
@@ -603,10 +603,13 @@ static void genmon_free (XfcePanelPlugin *plugin, genmon_t *poPlugin)
     TRACE ("genmon_free()\n");
     DBG("\n");
 
-    if (poPlugin->iTimerId)
+    if (poPlugin->iTimerId) {
         g_source_remove (poPlugin->iTimerId);
+        poPlugin->iTimerId = 0;
+    }
 
     g_free (poPlugin->oConf.oParam.acCmd);
+    g_free (poPlugin->oConf.oParam.acFiletmp);
     g_free (poPlugin->oConf.oParam.acTitle);
     g_free (poPlugin->oConf.oParam.acFont);
     g_free (poPlugin->oConf.oParam.acFonttmp);
@@ -851,10 +854,9 @@ static void SetPeriod (GtkWidget *p_wSc, void *p_pvPlugin)
 
 /**************************************************************/
 
-static void UpdateConf (void *p_pvPlugin)
+static void UpdateConf (struct genmon_t *poPlugin)
 /* Called back when the configuration/options window is closed */
 {
-    struct genmon_t *poPlugin = (genmon_t *) p_pvPlugin;
     struct conf_t  *poConf = &(poPlugin->oConf);
     struct gui_t   *poGUI = &(poConf->oGUI);
 
@@ -864,12 +866,7 @@ static void UpdateConf (void *p_pvPlugin)
     SetCmd (poGUI->wTF_Cmd, poPlugin);
     SetLabel (poGUI->wTF_Title, poPlugin);
     SetMonitorFont (poPlugin);
-    /* Restart timer */
-    if (poPlugin->iTimerId) {
-        g_source_remove (poPlugin->iTimerId);
-        poPlugin->iTimerId = 0;
-    }
-    SetTimer(p_pvPlugin);
+    SetTimer(poPlugin);
 }/* UpdateConf() */
 
 /**************************************************************/
@@ -888,7 +885,7 @@ static void About (XfcePanelPlugin *plugin)
 
   DBG("\n");
 
-  icon = xfce_panel_pixbuf_from_source ("utilities-system-monitor", NULL, 32);
+  icon = xfce_panel_pixbuf_from_source ("org.xfce.genmon", NULL, 32);
   gtk_show_about_dialog (NULL,
                          "logo",         icon,
                          "license",      xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
@@ -911,7 +908,7 @@ static void ChooseFont (GtkWidget *p_wPB, void *p_pvPlugin)
     struct genmon_t *poPlugin = (genmon_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     GtkWidget      *wDialog;
-    const char     *pcFont;
+    char           *pcFont;
     int             iResponse;
 
     DBG("\n");
@@ -929,6 +926,7 @@ static void ChooseFont (GtkWidget *p_wPB, void *p_pvPlugin)
             g_free (poConf->acFonttmp);
             poConf->acFonttmp = g_strdup (pcFont);
             gtk_button_set_label (GTK_BUTTON (p_wPB), poConf->acFonttmp);
+				g_free (pcFont);
         }
     }
     gtk_widget_destroy (wDialog);
@@ -936,20 +934,68 @@ static void ChooseFont (GtkWidget *p_wPB, void *p_pvPlugin)
 
 /**************************************************************/
 
+static void ChooseFile (GtkWidget *p_wPB, void *p_pvPlugin)
+{
+    struct genmon_t *poPlugin = (genmon_t *) p_pvPlugin;
+    struct param_t *poConf = &(poPlugin->oConf.oParam);
+	 struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
+
+    GtkWidget      *wDialog;
+    char           *pcFile;
+	 GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
+    int             iResponse;
+
+    DBG("\n");
+
+    wDialog = gtk_file_chooser_dialog_new (_("File Selection"),
+        GTK_WINDOW(gtk_widget_get_toplevel(p_wPB)), action, 
+		  _("_Cancel"), GTK_RESPONSE_CANCEL,
+        _("_Open"), GTK_RESPONSE_ACCEPT,
+        NULL);
+    gtk_window_set_transient_for (GTK_WINDOW (wDialog),
+        GTK_WINDOW (poPlugin->oConf.wTopLevel));
+    iResponse = gtk_dialog_run (GTK_DIALOG (wDialog));
+    if (iResponse == GTK_RESPONSE_ACCEPT) {
+        pcFile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (wDialog));
+        if (pcFile) {
+            g_free (poConf->acFiletmp);
+            poConf->acFiletmp = g_strdup (pcFile);
+			   gtk_entry_set_text (GTK_ENTRY (poGUI->wTF_Cmd), poConf->acFiletmp); 
+		  		g_free (pcFile);
+        }
+    }
+    gtk_widget_destroy (wDialog);
+}/* ChooseFile() */
+
+/**************************************************************/
+
 static void genmon_dialog_response (GtkWidget *dlg, int response,
     genmon_t *genmon)
 {
 	struct param_t *poConf = &(genmon->oConf.oParam);
     struct monitor_t *poMonitor = &(genmon->oMonitor);
+    gboolean result;
 
     DBG("\n");
 	
-	if (response == GTK_RESPONSE_OK) {
+    if (response == GTK_RESPONSE_HELP) {
+        result = g_spawn_command_line_async ("exo-open --launch WebBrowser " "https://docs.xfce.org/panel-plugins/xfce4-genmon-plugin", NULL);
+        if (G_UNLIKELY (result == FALSE))
+            g_warning (_("Unable to open the following url: %s"), "https://docs.xfce.org/panel-plugins/xfce4-genmon-plugin");
+        
+        return;
+    }
+	else if (response == GTK_RESPONSE_OK) {
 		if (poConf->acFonttmp)
 		{
 			g_free (poConf->acFont);
 		 	poConf->acFont = g_strdup (poConf->acFonttmp);
 		}
+		if (poConf->acFiletmp)
+		{
+			g_free (poConf->acCmd);
+		 	poConf->acCmd = g_strdup (poConf->acFiletmp);
+		}
 		
 	 	poConf->fTitleDisplayed = poConf->fTitleDisplayedtmp;
 	 	if (poConf->fTitleDisplayed)
@@ -966,6 +1012,7 @@ static void genmon_dialog_response (GtkWidget *dlg, int response,
 	}
 	else {
 		poConf->acFonttmp = g_strdup (poConf->acFont);
+		poConf->acFiletmp = g_strdup (poConf->acCmd);
 		poConf->fTitleDisplayedtmp = poConf->fTitleDisplayed;
 		poConf->iPeriod_mstmp = poConf->iPeriod_ms;
 	}
@@ -996,18 +1043,20 @@ static void genmon_create_options (XfcePanelPlugin *plugin,
     dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Generic Monitor"),
          GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
          GTK_DIALOG_DESTROY_WITH_PARENT,
+         "help-browser", _("_Help"), GTK_RESPONSE_HELP,
          "gtk-save", _("Save"), GTK_RESPONSE_OK,
          NULL);
 #else
     dlg = xfce_titled_dialog_new_with_buttons (_("Generic Monitor"),
         GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
         GTK_DIALOG_DESTROY_WITH_PARENT,
+        "help-browser", GTK_RESPONSE_HELP,
         "gtk-save", GTK_RESPONSE_OK,
         NULL);
 #endif
 
     gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
-    gtk_window_set_icon_name (GTK_WINDOW (dlg), "utilities-system-monitor");
+    gtk_window_set_icon_name (GTK_WINDOW (dlg), "org.xfce.genmon");
 
     g_signal_connect (dlg, "response", G_CALLBACK (genmon_dialog_response),
         poPlugin);
@@ -1035,6 +1084,9 @@ static void genmon_create_options (XfcePanelPlugin *plugin,
     g_signal_connect (GTK_WIDGET (poGUI->wTF_Cmd), "activate",
         G_CALLBACK (SetCmd), poPlugin);
 
+    g_signal_connect (G_OBJECT (poGUI->wPB_File), "clicked",
+		  G_CALLBACK (ChooseFile), poPlugin);
+
     gtk_entry_set_text (GTK_ENTRY (poGUI->wTF_Title), poConf->acTitle);
     g_signal_connect (GTK_WIDGET (poGUI->wTF_Title), "activate",
         G_CALLBACK (SetLabel), poPlugin);
@@ -1197,7 +1249,6 @@ static void genmon_construct (XfcePanelPlugin *plugin)
     gtk_container_add (GTK_CONTAINER (plugin), genmon->oMonitor.wEventBox);
 
     SetMonitorFont (genmon);
-    SetTimer (genmon);
 
     g_signal_connect (plugin, "free-data", G_CALLBACK (genmon_free), genmon);
 
@@ -1226,6 +1277,9 @@ static void genmon_construct (XfcePanelPlugin *plugin)
         G_CALLBACK (ExecOnValClickCmd), genmon);        
 
     xfce_panel_plugin_set_small (plugin, TRUE);
+
+    DisplayCmdOutput (genmon);
+    SetTimer (genmon);
 }
 
 XFCE_PANEL_PLUGIN_REGISTER (genmon_construct)
